//---------------------------------------------------------------------------
// BUFFER.CPP
//---------------------------------------------------------------------------

#include <buffer.hpp>
#include <mem.h>

//***************************************************************************
// BUFFER
//***************************************************************************

//---------------------------------------------------------------------------
   BUFFER::BUFFER (void)
//---------------------------------------------------------------------------
// Initialize to defaults
// A subsequent call to Alloc() will be required to set MaxLen
//---------------------------------------------------------------------------
{
   init();
}

//---------------------------------------------------------------------------
   BUFFER::BUFFER (UINT maxlen)
//---------------------------------------------------------------------------
// Initialize, then allocate the buffer
//---------------------------------------------------------------------------
{
   init(maxlen);
   Alloc();
}

//---------------------------------------------------------------------------
   BUFFER::BUFFER (BUFFER &b)
//---------------------------------------------------------------------------
// Make a copy of an existing buffer
//---------------------------------------------------------------------------
{
   init();
   copy(b);
}

//---------------------------------------------------------------------------
   BUFFER::~BUFFER (void)
//---------------------------------------------------------------------------
{
   Dealloc();
}

//---------------------------------------------------------------------------
   BUFFER & BUFFER::operator = (BUFFER &b)
//---------------------------------------------------------------------------
// Make a copy of an existing buffer
//---------------------------------------------------------------------------
{
   init();
   copy(b);
   return *this;
}

//---------------------------------------------------------------------------
   char & BUFFER::operator [] (UINT i)
//---------------------------------------------------------------------------
// Array index into buffer
//---------------------------------------------------------------------------
{
   if ((!Allocated) || (i >= CurLen)) {
      throw(BUFBNDS("Buffer Bounds Exceeded"));
      }
   return *(reinterpret_cast<LPSTR>(Buffer)+i);
}

//---------------------------------------------------------------------------
   void BUFFER::StreamIn (LPCSTR p, UINT len)
//---------------------------------------------------------------------------
// Stream data into the buffer
// Used by all operator<<()s
//---------------------------------------------------------------------------
{
   Realloc(CurLen + len);
   for (UINT i = 0; i < len; i++) {
      *(reinterpret_cast<LPSTR>(Buffer)+CurLen+i) = *(p++);
      }
   CurLen += len;
}

//---------------------------------------------------------------------------
   BUFFER & BUFFER::operator << (LPCSTR p)
//---------------------------------------------------------------------------
// Stream data into the buffer
//---------------------------------------------------------------------------
{
   StreamIn(p, strlen(p) + 1);
   return *this;
}

//---------------------------------------------------------------------------
   BUFFER & BUFFER::operator << (BOOL &p)
//---------------------------------------------------------------------------
// Stream data into the buffer
//---------------------------------------------------------------------------
{
   StreamIn(reinterpret_cast<LPSTR>(&p), sizeof(BOOL));
   return *this;
}

//---------------------------------------------------------------------------
   BUFFER & BUFFER::operator << (BYTE &p)
//---------------------------------------------------------------------------
// Stream data into the buffer
//---------------------------------------------------------------------------
{
   StreamIn(reinterpret_cast<LPSTR>(&p), sizeof(BYTE));
   return *this;
}

//---------------------------------------------------------------------------
   BUFFER & BUFFER::operator << (WORD &p)
//---------------------------------------------------------------------------
// Stream data into the buffer
//---------------------------------------------------------------------------
{
   StreamIn(reinterpret_cast<LPSTR>(&p), sizeof(WORD));
   return *this;
}

//---------------------------------------------------------------------------
   BUFFER & BUFFER::operator << (DWORD &p)
//---------------------------------------------------------------------------
// Stream data into the buffer
//---------------------------------------------------------------------------
{
   StreamIn(reinterpret_cast<LPSTR>(&p), sizeof(DWORD));
   return *this;
}

//---------------------------------------------------------------------------
   BUFFER & BUFFER::operator << (UINT &p)
//---------------------------------------------------------------------------
// Stream data into the buffer
//---------------------------------------------------------------------------
{
   StreamIn(reinterpret_cast<LPSTR>(&p), sizeof(UINT));
   return *this;
}

//---------------------------------------------------------------------------
   BUFFER & BUFFER::operator << (LONG &p)
//---------------------------------------------------------------------------
// Stream data into the buffer
//---------------------------------------------------------------------------
{
   StreamIn(reinterpret_cast<LPSTR>(&p), sizeof(LONG));
   return *this;
}

//---------------------------------------------------------------------------
   BUFFER & BUFFER::operator << (string &p)
//---------------------------------------------------------------------------
// Stream data into the buffer
//---------------------------------------------------------------------------
{
   if (p.length()) {
      StreamIn(p.c_str(), p.length()+1);
      }
   else {
      StreamIn("", 1);
      }
   return *this;
}

//---------------------------------------------------------------------------
   BUFFER & BUFFER::operator << (float &p)
//---------------------------------------------------------------------------
// Stream data into the buffer
//---------------------------------------------------------------------------
{
   StreamIn(reinterpret_cast<LPSTR>(&p), sizeof(float));
   return *this;
}

//---------------------------------------------------------------------------
   BUFFER & BUFFER::operator << (double &p)
//---------------------------------------------------------------------------
// Stream data into the buffer
//---------------------------------------------------------------------------
{
   StreamIn(reinterpret_cast<LPSTR>(&p), sizeof(double));
   return *this;
}

//---------------------------------------------------------------------------
   BUFFER & BUFFER::operator << (BUFFER &p)
//---------------------------------------------------------------------------
// Stream data into the buffer
//---------------------------------------------------------------------------
{
   StreamIn(reinterpret_cast<LPSTR>(&p), p.CurLen);
   return *this;
}

//---------------------------------------------------------------------------
   BUFFER & BUFFER::operator >> (string &p)
//---------------------------------------------------------------------------
// Stream data out of the buffer
//---------------------------------------------------------------------------
{
   LPSTR str = reinterpret_cast<LPSTR>(Buffer) + CurOut;
   p = str;
   CurOut += strlen(str) + 1;
   return *this;
}

//---------------------------------------------------------------------------
   void BUFFER::SetMaxLen (UINT len)
//---------------------------------------------------------------------------
// Allocate the buffer and update MaxLen
//---------------------------------------------------------------------------
{
   if (MaxLen != len) {
      Dealloc();
      MaxLen = len;
      Alloc();
      }
   else {
      CurLen = 0;
      }
}

//---------------------------------------------------------------------------
   void BUFFER::SetBuffer (LPVOID buf, UINT maxlen, UINT len)
//---------------------------------------------------------------------------
// Use a preallocated buffer
//---------------------------------------------------------------------------
{
   Dealloc();
   Buffer      = reinterpret_cast<LPSTR>(buf);
   MaxLen      = maxlen;
   CurLen      = len;
   CurOut      = 0;
}

//---------------------------------------------------------------------------
   void BUFFER::SetData (LPVOID buf, UINT len)
//---------------------------------------------------------------------------
// Move data into the buffer
//---------------------------------------------------------------------------
{
   if (len > MaxLen) {
      Dealloc();
      MaxLen = len;
      }
   if (!Allocated) {
      Alloc();
      }
   if (buf) {
      memmove(Buffer, buf, len);
      }
   else {
      memset(Buffer, NULL, len);
      }
   CurLen = len;
}

//---------------------------------------------------------------------------
   void BUFFER::GetData (LPVOID buf, UINT len)
//---------------------------------------------------------------------------
// Extract data from the buffer
//---------------------------------------------------------------------------
{
   memmove(buf, Buffer, len);
   CurOut = MaxLen;
}

//---------------------------------------------------------------------------
   void BUFFER::Alloc (void)
//---------------------------------------------------------------------------
// Allocate the buffer of MaxLen bytes
//---------------------------------------------------------------------------
{
   if (!Buffer) {
      Buffer      = new char[MaxLen];
      CurLen      = 0;
      CurOut      = 0;
      Allocated   = TRUE;
      }
}

//---------------------------------------------------------------------------
   void BUFFER::Realloc (UINT newlen)
//---------------------------------------------------------------------------
// Reallocate the data buffer
//---------------------------------------------------------------------------
{
   if ((Allocated) && (newlen > MaxLen)) {
      LPSTR    OldBuf   = (LPSTR)Buffer;
      UINT     OldLen   = CurLen;
      Allocated = FALSE; // so dealloc() won't delete OldBuf
      Dealloc();
      MaxLen = newlen;
      Alloc();
      SetData(OldBuf, OldLen);
      delete [] OldBuf;
      }
   else if (!Allocated) {
      MaxLen = newlen;
      Alloc();
      }
}

//---------------------------------------------------------------------------
   void BUFFER::Dealloc (void)
//---------------------------------------------------------------------------
// Deallocate the buffer
//---------------------------------------------------------------------------
{
   if ((Buffer) && (Allocated)) {
      delete [] (LPSTR)Buffer;
      }
   Buffer      = NULL;
   CurLen      = 0;
   CurOut      = 0;
   MaxLen      = 0;
   Allocated   = FALSE;
}

//---------------------------------------------------------------------------
   void BUFFER::init (UINT maxlen)
//---------------------------------------------------------------------------
// Initialize an instance
//---------------------------------------------------------------------------
{
   CurLen      = 0;
   CurOut      = 0;
   MaxLen      = maxlen;
   Buffer      = NULL;
   Allocated   = FALSE;
}

//---------------------------------------------------------------------------
   void BUFFER::copy (BUFFER &b)
//---------------------------------------------------------------------------
// Make a copy of an existing buffer
//---------------------------------------------------------------------------
{
   Dealloc();
   MaxLen      = b.MaxLen;
   Buffer      = NULL;
   Alloc();
   CurLen      = b.CurLen;
   CurOut      = 0;
   memcpy(Buffer, b.Buffer, CurLen);
}

